@@ -1,9 +1,12 @@
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 import pandas as pd
 
-from .alignment import align
-from .utils import Frozen, is_dict_like
-from .variable import as_variable, default_index_coordinate
-from .pycompat import (basestring, OrderedDict)
+from .alignment import deep_align
+from .pycompat import OrderedDict, basestring
+from .utils import Frozen
+from .variable import as_variable, assert_unique_multiindex_level_names
 
 
 PANDAS_TYPES = (pd.Series, pd.DataFrame, pd.Panel)
@@ -11,7 +14,8 @@ PANDAS_TYPES = (pd.Series, pd.DataFrame, pd.Panel)
 _VALID_COMPAT = Frozen({'identical': 0,
                         'equals': 1,
                         'broadcast_equals': 2,
-                        'minimal': 3})
+                        'minimal': 3,
+                        'no_conflicts': 4})
 
 
 def broadcast_dimension_size(variables):
@@ -47,7 +51,8 @@ def unique_variable(name, variables, compat='broadcast_equals'):
     variables : list of xarray.Variable
         List of Variable objects, all of which go by the same name in different
         inputs.
-    compat : {'identical', 'equals', 'broadcast_equals'}, optional
+    compat : {'identical', 'equals', 'broadcast_equals',
+              'no_conflicts'}, optional
         Type of equality check to use.
 
     Returns
@@ -60,12 +65,17 @@ def unique_variable(name, variables, compat='broadcast_equals'):
     """
     out = variables[0]
     if len(variables) > 1:
+        combine_method = None
+
         if compat == 'minimal':
             compat = 'broadcast_equals'
 
         if compat == 'broadcast_equals':
             dim_lengths = broadcast_dimension_size(variables)
-            out = out.expand_dims(dim_lengths)
+            out = out.set_dims(dim_lengths)
+
+        if compat == 'no_conflicts':
+            combine_method = 'fillna'
 
         for var in variables[1:]:
             if not getattr(out, compat)(var):
@@ -73,6 +83,11 @@ def unique_variable(name, variables, compat='broadcast_equals'):
                                  'objects to be combined:\n'
                                  'first value: %r\nsecond value: %r'
                                  % (name, out, var))
+            if combine_method:
+                # TODO: add preservation of attrs into fillna
+                out = getattr(out, combine_method)(var)
+                out.attrs = var.attrs
+
     return out
 
 
@@ -106,22 +121,20 @@ def merge_variables(
     ----------
     lists_of_variables_dicts : list of mappings with Variable values
         List of mappings for which each value is a xarray.Variable object.
-    priority_vars : mapping with Variable values, optional
+    priority_vars : mapping with Variable or None values, optional
         If provided, variables are always taken from this dict in preference to
         the input variable dictionaries, without checking for conflicts.
-    compat : {'identical', 'equals', 'broadcast_equals', 'minimal'}, optional
-        Type of equality check to use wben checking for conflicts.
+    compat : {'identical', 'equals', 'broadcast_equals',
+              'minimal', 'no_conflicts'}, optional
+        Type of equality check to use when checking for conflicts.
 
     Returns
     -------
-    OrderedDict keys given by the union of keys on list_of_variable_dicts
-    (unless compat=='minimal', in which case some variables with conflicting
-    values can be dropped), and Variable values corresponding to those that
-    should be found in the result.
+    OrderedDict with keys taken by the union of keys on list_of_variable_dicts,
+    and Variable values corresponding to those that should be found on the
+    merged result.
     """
     if priority_vars is None:
-        # one of these arguments (e.g., the first for in-place
-        # arithmetic or the second for Dataset.update) takes priority
         priority_vars = {}
 
     _assert_compat_valid(compat)
@@ -138,6 +151,8 @@ def merge_variables(
 
     for name, variables in lookup.items():
         if name in priority_vars:
+            # one of these arguments (e.g., the first for in-place arithmetic
+            # or the second for Dataset.update) takes priority
             merged[name] = priority_vars[name]
         else:
             dim_variables = [var for var in variables if (name,) == var.dims]
@@ -271,68 +286,23 @@ def coerce_pandas_values(objects):
     return out
 
 
-def merge_coords_without_align(objs, priority_vars=None):
+def merge_coords_for_inplace_math(objs, priority_vars=None):
     """Merge coordinate variables without worrying about alignment.
 
     This function is used for merging variables in coordinates.py.
     """
     expanded = expand_variable_dicts(objs)
     variables = merge_variables(expanded, priority_vars)
+    assert_unique_multiindex_level_names(variables)
     return variables
 
 
-def _align_for_merge(input_objects, join, copy, indexes=None):
-    """Align objects for merging, recursing into dictionary values.
-    """
-    if indexes is None:
-        indexes = {}
-
-    def is_alignable(obj):
-        return hasattr(obj, 'indexes') and hasattr(obj, 'reindex')
-
-    positions = []
-    keys = []
-    out = []
-    targets = []
-    no_key = object()
-    not_replaced = object()
-    for n, variables in enumerate(input_objects):
-        if is_alignable(variables):
-            positions.append(n)
-            keys.append(no_key)
-            targets.append(variables)
-            out.append(not_replaced)
-        else:
-            for k, v in variables.items():
-                if is_alignable(v) and k not in indexes:
-                    # Skip variables in indexes for alignment, because these
-                    # should to be overwritten instead:
-                    # https://github.com/pydata/xarray/issues/725
-                    positions.append(n)
-                    keys.append(k)
-                    targets.append(v)
-            out.append(OrderedDict(variables))
-
-    aligned = align(*targets, join=join, copy=copy, indexes=indexes)
-
-    for position, key, aligned_obj in zip(positions, keys, aligned):
-        if key is no_key:
-            out[position] = aligned_obj
-        else:
-            out[position][key] = aligned_obj
-
-    # something went wrong: we should have replaced all sentinel values
-    assert all(arg is not not_replaced for arg in out)
-
-    return out
-
-
 def _get_priority_vars(objects, priority_arg, compat='equals'):
     """Extract the priority variable from a list of mappings.
 
-    We need this method because in some cases the priority argument itself might
-    have conflicting values (e.g., if it is a dict with two DataArray values
-    with conflicting coordinate values).
+    We need this method because in some cases the priority argument itself
+    might have conflicting values (e.g., if it is a dict with two DataArray
+    values with conflicting coordinate values).
 
     Parameters
     ----------
@@ -340,7 +310,8 @@ def _get_priority_vars(objects, priority_arg, compat='equals'):
         Dictionaries in which to find the priority variables.
     priority_arg : int or None
         Integer object whose variable should take priority.
-    compat : 'broadcast_equals', 'equals' or 'identical', optional
+    compat : {'identical', 'equals', 'broadcast_equals',
+              'no_conflicts'}, optional
         Compatibility checks to use when merging variables.
 
     Returns
@@ -349,13 +320,24 @@ def _get_priority_vars(objects, priority_arg, compat='equals'):
     values indicating priority variables.
     """
     if priority_arg is None:
-        priority_vars = None
+        priority_vars = {}
     else:
         expanded = expand_variable_dicts([objects[priority_arg]])
         priority_vars = merge_variables(expanded, compat=compat)
     return priority_vars
 
 
+def expand_and_merge_variables(objs, priority_arg=None):
+    """Merge coordinate variables without worrying about alignment.
+
+    This function is used for merging variables in computation.py.
+    """
+    expanded = expand_variable_dicts(objs)
+    priority_vars = _get_priority_vars(objs, priority_arg)
+    variables = merge_variables(expanded, priority_vars)
+    return variables
+
+
 def merge_coords(objs, compat='minimal', join='outer', priority_arg=None,
                  indexes=None):
     """Merge coordinate variables.
@@ -366,10 +348,11 @@ def merge_coords(objs, compat='minimal', join='outer', priority_arg=None,
     """
     _assert_compat_valid(compat)
     coerced = coerce_pandas_values(objs)
-    aligned = _align_for_merge(coerced, join=join, copy=False, indexes=indexes)
+    aligned = deep_align(coerced, join=join, copy=False, indexes=indexes)
     expanded = expand_variable_dicts(aligned)
     priority_vars = _get_priority_vars(aligned, priority_arg, compat=compat)
     variables = merge_variables(expanded, priority_vars, compat=compat)
+    assert_unique_multiindex_level_names(variables)
 
     return variables
 
@@ -382,8 +365,12 @@ def merge_data_and_coords(data, coords, compat='broadcast_equals',
     return merge_core(objs, compat, join, explicit_coords=explicit_coords)
 
 
-def merge_core(objs, compat='broadcast_equals', join='outer', priority_arg=None,
-               explicit_coords=None, indexes=None):
+def merge_core(objs,
+               compat='broadcast_equals',
+               join='outer',
+               priority_arg=None,
+               explicit_coords=None,
+               indexes=None):
     """Core logic for merging labeled objects.
 
     This is not public API.
@@ -392,9 +379,10 @@ def merge_core(objs, compat='broadcast_equals', join='outer', priority_arg=None,
     ----------
     objs : list of mappings
         All values must be convertable to labeled arrays.
-    compat : 'broadcast_equals', 'equals' or 'identical', optional
+    compat : {'identical', 'equals', 'broadcast_equals',
+              'no_conflicts'}, optional
         Compatibility checks to use when merging variables.
-    join : 'outer', 'inner', 'left' or 'right', optional
+    join : {'outer', 'inner', 'left', 'right'}, optional
         How to combine objects with different indexes.
     priority_arg : integer, optional
         Optional argument in `objs` that takes precedence over the others.
@@ -421,7 +409,7 @@ def merge_core(objs, compat='broadcast_equals', join='outer', priority_arg=None,
     _assert_compat_valid(compat)
 
     coerced = coerce_pandas_values(objs)
-    aligned = _align_for_merge(coerced, join=join, copy=False, indexes=indexes)
+    aligned = deep_align(coerced, join=join, copy=False, indexes=indexes)
     expanded = expand_variable_dicts(aligned)
 
     coord_names, noncoord_names = determine_coords(coerced)
@@ -431,14 +419,13 @@ def merge_core(objs, compat='broadcast_equals', join='outer', priority_arg=None,
 
     priority_vars = _get_priority_vars(aligned, priority_arg, compat=compat)
     variables = merge_variables(expanded, priority_vars, compat=compat)
+    assert_unique_multiindex_level_names(variables)
 
     dims = calculate_dimensions(variables)
 
     for dim, size in dims.items():
-        if dim not in variables:
-            variables[dim] = default_index_coordinate(dim, size)
-
-    coord_names.update(dims)
+        if dim in variables:
+            coord_names.add(dim)
 
     ambiguous_coords = coord_names.intersection(noncoord_names)
     if ambiguous_coords:
@@ -449,7 +436,7 @@ def merge_core(objs, compat='broadcast_equals', join='outer', priority_arg=None,
     return variables, coord_names, dict(dims)
 
 
-def merge(objects, compat='broadcast_equals', join='outer'):
+def merge(objects, compat='no_conflicts', join='outer'):
     """Merge any number of xarray objects into a single Dataset as variables.
 
     Parameters
@@ -457,9 +444,20 @@ def merge(objects, compat='broadcast_equals', join='outer'):
     objects : Iterable[Union[xarray.Dataset, xarray.DataArray, dict]]
         Merge together all variables from these objects. If any of them are
         DataArray objects, they must have a name.
-    compat : 'broadcast_equals', 'equals' or 'identical', optional
-        Compatibility checks to use when merging variables.
-    join : 'outer', 'inner', 'left' or 'right', optional
+    compat : {'identical', 'equals', 'broadcast_equals',
+              'no_conflicts'}, optional
+        String indicating how to compare variables of the same name for
+        potential conflicts:
+
+        - 'broadcast_equals': all values must be equal when variables are
+          broadcast against each other to ensure common dimensions.
+        - 'equals': all values and dimensions must be the same.
+        - 'identical': all values, dimensions and attributes must be the
+          same.
+        - 'no_conflicts': only values which are not null in both datasets
+          must be equal. The returned dataset then contains the combination
+          of all non-null values.
+    join : {'outer', 'inner', 'left', 'right'}, optional
         How to combine objects with different indexes.
 
     Returns
@@ -503,8 +501,7 @@ def merge(objects, compat='broadcast_equals', join='outer'):
     return merged
 
 
-def dataset_merge_method(dataset, other, overwrite_vars=frozenset(),
-                         compat='broadcast_equals', join='outer'):
+def dataset_merge_method(dataset, other, overwrite_vars, compat, join):
     """Guts of the Dataset.merge method."""
 
     # we are locked into supporting overwrite_vars for the Dataset.merge
